/*--------------------------------------------------------------------------
     "libfunc.h" - library functions and classes
        This is a part of KUCRS:
           Knowledge-basing Utilities for Complex Reaction Systems
                        (c) 2002-2023 by A. Miyoshi, Hiroshima University
                                       created: Aug.   5, 2002
                                        edited: May    1, 2004
                                        edited: Nov.  28, 2011
                                        edited: Dec.   4, 2015
                                        edited: June   6, 2017
                                        edited: Dec.  26, 2021
                                        edited: July  20, 2022
                                   last edited: Apr.  20, 2023
--------------------------------------------------------------------------*/

#ifndef _LIBFUNC_H_
#define _LIBFUNC_H_

#ifndef _CRT_SECURE_NO_WARNINGS
#define _CRT_SECURE_NO_WARNINGS
#endif

#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <algorithm>
#include <vector>
#include <deque>
#include <list>
#include <set>
#include <map>
#include <cctype>
#include <cstring>
#include <cstdlib>
#include <cmath>
#include <ctime>
using namespace std;

#define NORM 0
#define ERRE (-1)
// - these are obsolute for C++
// #ifndef true
// #define true (-1)
// #endif
// #ifndef false
// #define false 0
// #endif

#define ENVVARNAM "KUCRS_LIB"           // environment variable name
#define KUCRS_REV "KUCRS rev. 2023.04.20"
#define COPYR_STR "(c) 2002-2023 by A. Miyoshi"

// ===== general purpose functions and classes =============================

string strToUpper(const string &s);
string strToLower(const string &s);
string format(int v);
string format(int v, int len);
string format(string s, int len);
string formatLeft(string s, int len);
string formatRight(string s, int len);
string formatE(double v, int len, int prc);
string formatF(double v, int len, int prc);
string formatFnoDot(double v, int len);
string formatFautoLen(double v, int prc);
string formatD(int v, int len);
string formatBool(int flag);
int valBoolStr(string s);
long valHexStr(string s);
string strCurDate();
int nextCorCket(string &s, int start);
int numMatchStrList(string s, vector<string> *vs);
ostream &operator<<(ostream &os, vector<string> &vs);
int splitToken(const string &s, const string &spl, vector<string> &vs);
string strTermBy(const string &s, char ct);
string strRemoveSpc(const string &s);
int getOneLine(ifstream &inf, string &s);
int getTokensFrom(ifstream &inf, int ntkn, vector<string> &vs);
int getTokenFrom(ifstream &inf, string &s);
int getCharsFile(string flname, string &s);
string getBaseName(string &s);
string getBasePartEndWith(string nam, string strEnd);
int getGCM(int x, int y);
int getLCM(int x, int y);
int copyFile(string flnin, string flnout);
int addToFile(string flnin1, string flnin2, string flnout);
int add3Files(string flnin1, string flnin2, string flnin3, string flnout);

// ===== library functions and classes for KUCRS ===========================

void showRevision();
void printHeadingInfo(ostream &os);

// ----- libPath class ---------------------------------------------------

class libPath {
 private:                                        // --- data members ---
  string pathStr;
 public:                                     // --- member functions ---
  libPath();
  libPath(const string &namEnvVar);
  void init(const string &namEnvVar);
  string fullPath(string libname);
};

// ----- macroMap and related classes ------------------------------------

class macroMap : public map<string, string> {
 private:                                        // --- data members ---
  string alwdChars;
  string alwdNonTop;
 public:                                     // --- member functions ---
  macroMap();
  macroMap(string alwdch, string anontop);
  void init(string alwdch, string anontop);
  int loadFrom(string flname, libPath &libp);
  string getSubsVal(string key);
  int substMacros(string &tgs);
 private:
  int isAllowed(char c);
  int isAllowedTop(char c);
};

// ----- outStrms class --------------------------------------------------

class outStrms {
 public:                                         // --- data members ---
  ofstream thm, rxn, mol, mll;
 public:                                     // --- member functions ---
  int open(string fnt, string fnr, string fnm, string fnl);
  void close();
};

#endif
